@@ -344,6 +344,12 @@ int display_configure(const char *config)
             return ret;
     }
 
+    if (!display_is_external) {
+        // Sonoma bug workaround: Power on internal panel early
+        if ((ret = dcp_ib_set_power(iboot, true)) < 0)
+            printf("display: failed to set power on (continuing anyway)\n");
+    }
+
     // Detect if display is connected
     int timing_cnt, color_cnt;
     int hpd = 0, retries = 0;
@@ -378,6 +384,9 @@ int display_configure(const char *config)
         return ret;
     }
 
+    // Sonoma bug workaround
+    mdelay(100);
+
     // Find best modes
     dcp_timing_mode_t *tmodes, tbest;
     if ((ret = dcp_ib_get_timing_modes(iboot, &tmodes)) < 0) {
@@ -397,8 +406,12 @@ int display_configure(const char *config)
 
     // Set mode
     if ((ret = dcp_ib_set_mode(iboot, &tbest, &cbest)) < 0) {
-        printf("display: failed to set mode\n");
-        return -1;
+        printf("display: failed to set mode. trying again...\n");
+        mdelay(500);
+        if ((ret = dcp_ib_set_mode(iboot, &tbest, &cbest)) < 0) {
+            printf("display: failed to set mode twice.\n");
+            return ret;
+        }
     }
 
     u64 fb_pa = cur_boot_args.video.base;
@@ -475,6 +488,11 @@ int display_configure(const char *config)
 
     printf("display: swapped! (swap_id=%d)\n", ret);
 
+    // Wait until the swap completes before powering down DCP
+    // 50ms is too low, 100 works, 150 for good measure
+    mdelay(150);
+
+    bool reinit = false;
     if (fb_pa != cur_boot_args.video.base || cur_boot_args.video.stride != stride ||
         cur_boot_args.video.width != tbest.width || cur_boot_args.video.height != tbest.height ||
         cur_boot_args.video.depth != 30) {
@@ -483,9 +501,17 @@ int display_configure(const char *config)
         cur_boot_args.video.width = tbest.width;
         cur_boot_args.video.height = tbest.height;
         cur_boot_args.video.depth = 30 | (opts.retina ? FB_DEPTH_FLAG_RETINA : 0);
-        fb_reinit();
+        reinit = true;
     }
 
+    if (!display_is_external && !(cur_boot_args.video.depth & FB_DEPTH_FLAG_RETINA)) {
+        cur_boot_args.video.depth |= FB_DEPTH_FLAG_RETINA;
+        reinit = true;
+    }
+
+    if (reinit)
+        fb_reinit();
+
     /* Update for python / subsequent stages */
     memcpy((void *)boot_args_addr, &cur_boot_args, sizeof(cur_boot_args));
 
@@ -528,6 +554,11 @@ int display_init(void)
     } else if (display_is_external) {
         printf("display: External display found, reconfiguring\n");
         return display_configure(NULL);
+    } else if (!(cur_boot_args.video.depth & FB_DEPTH_FLAG_RETINA)) {
+        printf("display: Internal display with non-retina flag, assuming Sonoma bug and "
+               "reconfiguring\n");
+        fb_clear_direct(); // Old m1n1 stage1 ends up with an ugly logo situation, clear it.
+        return display_configure(NULL);
     } else {
         printf("display: Display is already initialized (%ldx%ld)\n", cur_boot_args.video.width,
                cur_boot_args.video.height);
